__version__ = '0.1.4'

import traceback
from importlib import import_module
from pathlib import Path


MODULES = {
	# misc
	'Digest': 'misc',
	'Dotdict': 'misc',
	'HttpDate': 'misc',
	'Signature': 'misc',

	#objects
	'HostMeta': 'objects',
	'HostMetaJson': 'objects',
	'Nodeinfo': 'objects',
	'Webfinger': 'objects',
	'WellKnownNodeinfo': 'objects',

	# signer
	'Signer': 'signer',

	# enums
	'Enum': 'enums',
	'IntEnum': 'enums',
	'StrEnum': 'enums',
	'AlgorithmType': 'enums',
	'KeyType': 'enums',
	'NodeinfoProtocol': 'enums',
	'NodeinfoServiceInbound': 'enums',
	'NodeinfoServiceOutbound': 'enums',
	'ObjectType': 'enums',
	'ObjectTypeMeta': 'enums',

	# exceptions
	'SignatureFailureError': 'errors'
}


## lazy import
def __getattr__(key):
	try: return globals()[key]
	except KeyError: pass

	try:
		module = import_module(f'aputils.{MODULES[key]}')
		return getattr(module, key)

	except ModuleNotFoundError:
		traceback.print_exc()
		raise ImportError(f'Cannot import "{key}" from "aputils.{MODULES[key]}"') from None

	except KeyError:
		pass

	except Exception as e:
		traceback.print_exc()
		raise e

	return import_module(f'aputils.{key}')


def get_hook_dirs():
	return [str(Path(__file__).resolve().parent)]
